/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <unistd.h>
# include <getopt.h>
# include <sys/param.h>
# include "fnode.h"
# include "explode.h"
# include "pkg_add_it.h"
# include "scan_directory.h"
# include "scan_index_file.h"

int main (int argc, char *argv[])
{
   char *cwd;
   bool_t local, foundopt;
   int chopt, numfiles, index;
   fnode_t *head, *pkgfile;
   
   /* options struct */
   static struct option longopts[] = {
      { "version", no_argument, NULL, 'v' },
      { "help", no_argument, NULL, 'h' },
      { "local", no_argument, NULL, 'l' },
      { "remote", no_argument, NULL, 'r' },
      { NULL, 0, NULL, 0 }
   } ;
        
   index = numfiles = 0;  
   local = foundopt = FALSE;
   
   /* save the current dir */
   if (!(cwd = getcwd(NULL, MAXPATHLEN + 1))) {
      perror ("getcwd");
      exit (1);
   }
   
   /* check for valid command-line options */
   while (!foundopt && ((chopt = getopt_long(argc, argv, "vhl:r:", longopts, NULL)) != -1)) {
      switch (chopt) {
         case 'h':
            usage();
            exit (0);
            break;   /* never reached - added only to follow standarts */
         case 'v':
         # ifdef __STDC__
            fprintf (stdout, "==>> %s v%s : built on %s at %s\n", PROGNAME, PROGVERS, __DATE__, __TIME__);
         # endif
            exit (0);
            break;   /* never reached - added only to follow standarts */
         case 'l':   
            /* make some checks first */
            if (argc < 4) {
               fprintf (stderr, "==>> You have to specify [pkg-dir] and [pkg-name]\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            else if (argc > 4) {
               fprintf (stderr, "==>> Too many arguments passed\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            if (!dir_exists(argv[2])) {
               fprintf (stderr, "==>> '%s' is not a directory\n", argv[2]);
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            
            /* perform a local pattern search */
            local = foundopt = TRUE;
                        
            fnode_push (&head, argv[3], argv[2], NULL, NULL);
            head->next = NULL;
            numfiles = scan_directory (argv[2], argv[3], &head);
            break;
         case 'r':
            if (argc < 3) {
               fprintf (stderr, "==>> You have to specify [pkg-name]\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            else if (argc > 3) {
               fprintf (stderr, "==>> Too many arguments passed\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            
            /* perform a remote pattern search */
            foundopt = TRUE;
            
            fnode_push (&head, argv[2], ".", NULL, NULL);
            head->next = NULL;
            numfiles = scan_index_file (argv[2], &head);
            break;
         default:
            exit (1);   /* no valid options were found */
      }
   }     
   
   /* check if there were any command line options at all */
   if (optind == 1) {
      fprintf (stderr, "==>> Invalid call to %s\n", PROGNAME);
      fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
      exit (1);
   }        
   
   if (!numfiles) {
      fprintf (stderr, "==>> No matching packages were found..\n");
      fprintf (stderr, "==>> Perhaps a wrong package name given?\n");
   }
   
   else {
      index = fnode_display (head, numfiles);
      fnode_locate (head, index, &pkgfile);
      install_cmd (pkgfile, local);
   }
   
   /* return to where we were */
   chdir (cwd);
   
   /* deallocate the stack */
   fnode_shutdown (head);
   
   return 0;
}

/* prints the usage of the program */
void usage (USAGEARGS)
{
   fprintf (stderr, "Usage: %s [options] [pkg-dir] [pkg-name]\n\n", PROGNAME);
   fprintf (stderr, "%s - %s, v%s\n", PROGNAME, PROGDESC, PROGVERS);
# ifdef __STDC__
   printf ("%s v%s : built on %s at %s\n", PROGNAME, PROGVERS, __DATE__, __TIME__);
# endif
   fprintf (stderr, "\nAvailable options:\n");
   fprintf (stderr, "--version|-v\tPrints the current version of the program\n");
   fprintf (stderr, "--help|-h\tDisplays this help info\n");
   fprintf (stderr, "--local|-l\tSpecifies a local pattern search\n");
   fprintf (stderr, "--remote|-r\tSpecifies a remote pattern search\n");
   fprintf (stderr, "[pkg-dir]\tSpecifies the directory containing package files (.tbz)\n");
   fprintf (stderr, "\t\tUsed only with the `--local` option\n");
   fprintf (stderr, "[pkg-name]\tSpecifies the package name\n");
   fprintf (stderr, "\nExample usage:\n");
   fprintf (stderr, "To perform a local package search for `kde' use this:\n");
   fprintf (stderr, "`%s --local /cdrom/packages/All kde`\n", PROGNAME);
   fprintf (stderr, "\nTo perform a remote package search for `xorg` use this:\n");
   fprintf (stderr, "`%s --remote kde`\n", PROGNAME);
   fprintf (stderr, "\nFor more information read pkg_add_it(1)\n");
}

/* installs the chose package */
void install_cmd (INSTALLCMDARGS)
{
   char *category, *ftppath, buf[MAXPATHLEN + 1];
   explode_t *expl_head, *expl_tail;
   int expl_count = 0;
   
   /* local installation */
   if (local)
      snprintf (buf, sizeof(buf), "%s %s/%s", CMDLOCAL, pkgfile->fpath, pkgfile->fname);
   /* remote installation */
   else {
      explode_push_queue (pkgfile->fpath, &expl_head);
      expl_tail = expl_head;
      expl_count = explode (' ', pkgfile->fpath, &expl_tail);
      
      ftppath = build_ftp_path ();
      category = (expl_count == 0) ? expl_head->str : expl_head->next->str;
      
      explode_destroy_queue (expl_head);
      
      snprintf (buf, sizeof(buf), "%s %s%s/%s.tbz", CMDREMOTE, ftppath, category, pkgfile->fname);
   }
     
   system (buf);
}
